/*
 GBxCart RW - GB Camera Save to BMP
 Version: 1.3
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 8/07/2017
 Last Modified: 30/03/2018
 
 GBxCart RW allows you to dump your Gameboy/Gameboy Colour/Gameboy Advance games ROM, save the RAM and write to the RAM.
 
 This is a special build which extracts your GB camera images and converts them to BMP files. 
 We create a folder with the date/time and save each image to a BMP file in that folder.
  
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include <time.h>

#ifdef _WIN32
#include <Windows.h>
#else
#include <unistd.h>
#endif

#include "setup.h" // See defines, variables, constants, functions here

int main(int argc, char **argv) {
	
	printf("GBxCart RW - GB Camera Saver v1.3 by insideGadgets\n");
	printf("##################################################\n");
	
	// Check arguments
	if (argc >= 2) {
		read_config_folder();
		
		// Get the date/time to be used for the folder
		time_t rawtime;
		struct tm* timeinfo;
		char timebuffer[25];
		
		time (&rawtime);
		timeinfo = localtime (&rawtime);
		strftime (timebuffer, 80, "%Y-%m-%d_%H-%M-%S", timeinfo);
		
		// Create folder
		char mkfolder[250];
		strncpy(mkfolder, "mkdir ", 7);
		strncat(mkfolder, currentFolder, 219);
		strncat(mkfolder, timebuffer, 25);
		system(mkfolder);
		
		// Convert .sav file to BMP image files
		printf("\n\nExtracting images from .sav file and converting to BMP images");
		
		// Locate which photos aren't marked as deleted
		uint16_t activePhotosLocation = 0x11B2;
		uint8_t activePhotosReadCount = 0x1E;
		uint8_t activePhotos[30];
		
		FILE *saveFile = fopen(argv[1], "rb");
		fseek(saveFile, activePhotosLocation, SEEK_SET);
		fread(activePhotos, 1, activePhotosReadCount, saveFile);
		fclose(saveFile);
		
		// Only save photos which are actively showing (i.e not marked as 0xFF)
		uint8_t saveName = 1;
		for (uint8_t x = 0; x < 30; x++) {
			if (activePhotos[x] != 0xFF) {
				uint32_t startLocation = (x * 0x1000) + 0x2000;
				uint16_t startReadCount = 3584;
				
				// Read save file
				FILE *saveFile = fopen(argv[1], "rb");
				fseek(saveFile, startLocation, SEEK_SET);
				fread(savBuffer, 1, startReadCount, saveFile);
				fclose(saveFile);
				
				// Write to BMP
				char bmpName[60];
				sprintf(bmpName, "%s%s/%i.bmp", currentFolder, timebuffer, saveName);
				save_to_bmp(bmpName);
				saveName++;
			}
		}
		
		printf("\nFinished\n");
	}
	else {
		printf("\nPlease drag and drop your GB Camera save file.");
		read_one_letter();
		return 0;
	}
	
	return 0;
}